@@ -1,15 +1,9 @@
-_base_ = 'configs/base.py'
-
-log_config = dict(
-    interval=10,
-    hooks=[dict(type='TextLoggerHook'),
-           dict(type='TensorboardLoggerHook')])
+_base_ = '../common/dataset/imagenet_classification.py'
 
 # model settings
 model = dict(
     type='Classification',
     train_preprocess=['mixUp'],
-    pretrained=False,
     mixup_cfg=dict(
         mixup_alpha=0.8,
         cutmix_alpha=1.0,
@@ -29,17 +23,11 @@ model = dict(
         },
         with_fc=False))
 
-data_train_list = 'data/imagenet_raw/meta/train_labeled.txt'
-data_train_root = 'data/imagenet_raw/train/'
-data_test_list = 'data/imagenet_raw/meta/val_labeled.txt'
-data_test_root = 'data/imagenet_raw/val/'
-data_all_list = 'data/imagenet_raw/meta/all_labeled.txt'
-data_root = 'data/imagenet_raw/'
-
-dataset_type = 'ClsDataset'
+image_size2 = 224
 img_norm_cfg = dict(mean=[0.485, 0.456, 0.406], std=[0.229, 0.224, 0.225])
+
 train_pipeline = [
-    dict(type='RandomResizedCrop', size=224),
+    dict(type='RandomResizedCrop', size=image_size2),
     dict(type='RandomHorizontalFlip'),
     dict(
         type='MMRandAugment',
@@ -62,44 +50,11 @@ train_pipeline = [
     dict(type='Normalize', **img_norm_cfg),
     dict(type='Collect', keys=['img', 'gt_labels'])
 ]
-test_pipeline = [
-    dict(type='Resize', size=256),
-    dict(type='CenterCrop', size=224),
-    dict(type='ToTensor'),
-    dict(type='Normalize', **img_norm_cfg),
-    dict(type='Collect', keys=['img', 'gt_labels'])
-]
 
 data = dict(
     imgs_per_gpu=64,  # total 256
     workers_per_gpu=8,
-    train=dict(
-        type=dataset_type,
-        data_source=dict(
-            list_file=data_train_list,
-            root=data_train_root,
-            type='ClsSourceImageList'),
-        pipeline=train_pipeline),
-    val=dict(
-        type=dataset_type,
-        data_source=dict(
-            list_file=data_test_list,
-            root=data_test_root,
-            type='ClsSourceImageList'),
-        pipeline=test_pipeline))
-
-eval_config = dict(initial=True, interval=1, gpu_collect=True)
-eval_pipelines = [
-    dict(
-        mode='test',
-        data=data['val'],
-        dist_eval=True,
-        evaluators=[dict(type='ClsEvaluator', topk=(1, 5))],
-    )
-]
-
-# additional hooks
-custom_hooks = []
+    train=dict(pipeline=train_pipeline))
 
 # optimizer
 paramwise_options = {
