@@ -156,8 +156,6 @@ eval_pipelines = [
 ]
 
 checkpoint_config = dict(interval='${interval}')
-# export model during training
-checkpoint_sync_export = True
 # optimizer
 # basic_lr_per_img = 0.01 / 64.0
 optimizer = dict(
@@ -197,8 +195,6 @@ load_from = None
 resume_from = None
 workflow = [('train', 1)]
 
-export = dict(use_jit=False)
-
 # oss io config
 oss_io_config = dict(
     ak_id='your oss ak id',
@@ -206,3 +202,7 @@ oss_io_config = dict(
     hosts='oss-cn-zhangjiakou.aliyuncs.com',
     buckets=['your_bucket'])
 oss_sync_config = dict(other_file_list=['**/events.out.tfevents*', '**/*log*'])
+
+# export model during training
+checkpoint_sync_export = True
+export = dict(export_type='raw')
