@@ -43,6 +43,10 @@ CLASSES = [
 
 # dataset settings
 data_root = 'data/coco/'
+train_ann_file = data_root + 'annotations/instances_train2017.json'
+train_img_prefix = data_root + 'train2017/'
+test_ann_file = data_root + 'annotations/instances_val2017.json'
+test_img_prefix = data_root + 'val2017/'
 
 img_norm_cfg = dict(
     mean=[123.675, 116.28, 103.53], std=[58.395, 57.12, 57.375], to_rgb=True)
@@ -83,8 +87,8 @@ train_dataset = dict(
     type='DetImagesMixDataset',
     data_source=dict(
         type='DetSourceCoco',
-        ann_file=data_root + 'annotations/instances_train2017.json',
-        img_prefix=data_root + 'train2017/',
+        ann_file=train_ann_file,
+        img_prefix=train_img_prefix,
         pipeline=[
             dict(type='LoadImageFromFile', to_float32=True),
             dict(type='LoadAnnotations', with_bbox=True)
@@ -100,8 +104,8 @@ val_dataset = dict(
     imgs_per_gpu=2,
     data_source=dict(
         type='DetSourceCoco',
-        ann_file=data_root + 'annotations/instances_val2017.json',
-        img_prefix=data_root + 'val2017/',
+        ann_file=test_ann_file,
+        img_prefix=test_img_prefix,
         pipeline=[
             dict(type='LoadImageFromFile', to_float32=True),
             dict(type='LoadAnnotations', with_bbox=True)
@@ -188,4 +192,5 @@ log_config = dict(
         # dict(type='WandbLoggerHookV2'),
     ])
 
+checkpoint_sync_export = True
 export = dict(export_type = 'raw', preprocess_jit = False, batch_size=1, blade_config=dict(enable_fp16=True, fp16_fallback_op_ratio=0.01), use_trt_efficientnms=False)
