@@ -1,4 +1,7 @@
 # Copyright (c) Alibaba, Inc. and its affiliates.
+from .box3d_nms import (aligned_3d_nms, box3d_multiclass_nms, circle_nms,
+                        nms_bev, nms_normal_bev)
+from .merge_augs import merge_aug_bboxes_3d
 from .nms import oks_nms, soft_oks_nms
 from .pose_transforms import (affine_transform, flip_back, fliplr_joints,
                               fliplr_regression, get_affine_transform,
