@@ -1,6 +1,8 @@
 # Copyright (c) OpenMMLab. All rights reserved.
 # Copyright (c) Alibaba, Inc. and its affiliates.
+import concurrent.futures
 import copy
+import logging
 import random
 import tempfile
 from os import path as osp
@@ -14,6 +16,7 @@ from easycv.core.bbox import Box3DMode, Coord3DMode
 from easycv.datasets.registry import DATASETS
 from easycv.datasets.shared.base import BaseDataset
 from easycv.datasets.shared.pipelines import Compose
+from easycv.datasets.shared.pipelines.format import to_tensor
 from .utils import extract_result_dict
 
 
@@ -50,6 +53,7 @@ class NuScenesDataset(BaseDataset):
         self.eval_detection_configs = config_factory(self.eval_version)
         self.flag = np.zeros(
             len(self), dtype=np.uint8)  # for DistributedGroupSampler
+        self.pipeline_cfg = pipeline
 
     def _format_bbox(self, results, jsonfile_prefix=None):
         """Convert the results to the standard format.
@@ -309,6 +313,9 @@ class NuScenesDataset(BaseDataset):
         prev_scene_token = None
         prev_pos = None
         prev_angle = None
+
+        can_bus_list = []
+        lidar2img_list = []
         for i, each in enumerate(queue):
             metas_map[i] = each['img_metas'].data
             if metas_map[i]['scene_token'] != prev_scene_token:
@@ -326,28 +333,75 @@ class NuScenesDataset(BaseDataset):
                 metas_map[i]['can_bus'][-1] -= prev_angle
                 prev_pos = copy.deepcopy(tmp_pos)
                 prev_angle = copy.deepcopy(tmp_angle)
+
+            can_bus_list.append(to_tensor(metas_map[i]['can_bus']))
+            lidar2img_list.append(to_tensor(metas_map[i]['lidar2img']))
+
         queue[-1]['img'] = DC(
             torch.stack(imgs_list), cpu_only=False, stack=True)
         queue[-1]['img_metas'] = DC(metas_map, cpu_only=True)
+        queue[-1]['can_bus'] = DC(torch.stack(can_bus_list), cpu_only=False)
+        queue[-1]['lidar2img'] = DC(
+            torch.stack(lidar2img_list), cpu_only=False)
         queue = queue[-1]
         return queue
 
+    @staticmethod
+    def _get_single_data(i,
+                         data_source,
+                         pipeline,
+                         flip_flag=False,
+                         scale=None):
+        i = max(0, i)
+        try:
+            data = data_source[i]
+            data['flip_flag'] = flip_flag
+            if scale:
+                data['resize_scale'] = scale
+            data = pipeline(data)
+            if data is None or ~(data['gt_labels_3d']._data != -1).any():
+                return None
+        except Exception as e:
+            logging.error(e)
+            return None
+        return i, data
+
     def _get_queue_data(self, idx):
         queue = []
         idx_list = list(range(idx - self.queue_length, idx))
         random.shuffle(idx_list)
         idx_list = sorted(idx_list[1:])
         idx_list.append(idx)
-        for i in idx_list:
-            i = max(0, i)
-            try:
-                data = self.data_source[i]
-                data = self.pipeline(data)
-                if data is None or ~(data['gt_labels_3d']._data != -1).any():
-                    return None
-            except Exception as e:
-                return None
-            queue.append(data)
+
+        flip_flag = False
+        scale = None
+        for member in self.pipeline_cfg:
+
+            if member['type'] == 'RandomScaleImageMultiViewImage':
+                scales = member['scales']
+                rand_ind = np.random.permutation(range(len(scales)))[0]
+                scale = scales[rand_ind]
+            if member['type'] == 'RandomHorizontalFlipMultiViewImage':
+                flip_flag = np.random.rand() >= 0.5
+
+        with concurrent.futures.ThreadPoolExecutor(
+                max_workers=len(idx_list)) as executor:
+            threads = []
+            for i in idx_list:
+                future = executor.submit(self._get_single_data, i,
+                                         self.data_source, self.pipeline,
+                                         flip_flag, scale)
+                threads.append(future)
+
+            for future in concurrent.futures.as_completed(threads):
+                queue.append(future.result())
+
+        if None in queue:
+            return None
+
+        queue = sorted(queue, key=lambda item: item[0])
+        queue = [item[1] for item in queue]
+
         return self.union2one(queue)
 
     def __getitem__(self, idx):
@@ -358,6 +412,18 @@ class NuScenesDataset(BaseDataset):
                 data_dict = self.data_source[idx]
                 data_dict = self.pipeline(data_dict)
 
+                can_bus_list, lidar2img_list = [], []
+                for i in range(len(data_dict['img_metas'])):
+                    can_bus_list.append(
+                        to_tensor(data_dict['img_metas'][i]._data['can_bus']))
+                    lidar2img_list.append(
+                        to_tensor(
+                            data_dict['img_metas'][i]._data['lidar2img']))
+                data_dict['can_bus'] = DC(
+                    torch.stack(can_bus_list), cpu_only=False)
+                data_dict['lidar2img'] = DC(
+                    torch.stack(lidar2img_list), cpu_only=False)
+
             if data_dict is None:
                 idx = self._rand_another(idx)
                 continue
