@@ -1,16 +1,23 @@
 # Copyright (c) Alibaba, Inc. and its affiliates.
 from __future__ import division
 import math
-import os
 import random
 
 import numpy as np
 import torch
+import torch.distributed as dist
 from mmcv.runner import get_dist_info
 from torch.utils.data import DistributedSampler as _DistributedSampler
-from torch.utils.data import Sampler
+from torch.utils.data import RandomSampler, Sampler
+
+from easycv.datasets.registry import SAMPLERS
+from easycv.framework.errors import ValueError
+from easycv.utils.dist_utils import local_rank
+
+SAMPLERS.register_module(RandomSampler)
 
 
+@SAMPLERS.register_module()
 class DistributedMPSampler(_DistributedSampler):
 
     def __init__(self,
@@ -18,7 +25,8 @@ class DistributedMPSampler(_DistributedSampler):
                  num_replicas=None,
                  rank=None,
                  shuffle=True,
-                 split_huge_listfile_byrank=False):
+                 split_huge_listfile_byrank=False,
+                 **kwargs):
         """ A Distribute sampler which support sample m instance from one class once for classification dataset
             dataset: pytorch dataset object
             num_replicas (optional): Number of processes participating in
@@ -30,9 +38,7 @@ class DistributedMPSampler(_DistributedSampler):
         """
         super().__init__(dataset, num_replicas=num_replicas, rank=rank)
 
-        current_env = os.environ.copy()
-        self.local_rank = int(current_env['LOCAL_RANK'])
-
+        self.local_rank = local_rank()
         self.shuffle = shuffle
         self.unif_sampling_flag = False
         self.split_huge_listfile_byrank = split_huge_listfile_byrank
@@ -83,7 +89,9 @@ class DistributedMPSampler(_DistributedSampler):
         self.label_list = []
 
         if not self.dataset.data_source.has_labels:
-            raise 'MPSampler need initial with classification datasets which has label!'
+            raise ValueError(
+                'MPSampler need initial with classification datasets which has label!'
+            )
 
         for idx, label in enumerate(self.dataset.data_source.labels):
             if label in self.label_dict.keys():
@@ -153,6 +161,7 @@ class DistributedMPSampler(_DistributedSampler):
         return self.length
 
 
+@SAMPLERS.register_module()
 class DistributedSampler(_DistributedSampler):
 
     def __init__(
@@ -251,6 +260,7 @@ class DistributedSampler(_DistributedSampler):
         return self.num_samples if not self.split_huge_listfile_byrank else self.num_samples * self.num_replicas
 
 
+@SAMPLERS.register_module()
 class GroupSampler(Sampler):
 
     def __init__(self, dataset, samples_per_gpu=1):
@@ -292,6 +302,7 @@ class GroupSampler(Sampler):
         return self.num_samples
 
 
+@SAMPLERS.register_module()
 class DistributedGroupSampler(Sampler):
     """Sampler that restricts data loading to a subset of the dataset.
     It is especially useful in conjunction with
@@ -384,6 +395,7 @@ class DistributedGroupSampler(Sampler):
         self.epoch = epoch
 
 
+@SAMPLERS.register_module()
 class DistributedGivenIterationSampler(Sampler):
 
     def __init__(self,
@@ -469,3 +481,75 @@ class DistributedGivenIterationSampler(Sampler):
 
     def set_epoch(self, epoch):
         pass
+
+
+@SAMPLERS.register_module()
+class RASampler(torch.utils.data.Sampler):
+    """Sampler that restricts data loading to a subset of the dataset for distributed,
+    with repeated augmentation.
+    It ensures that different each augmented version of a sample will be visible to a
+    different process (GPU)
+    Heavily based on torch.utils.data.DistributedSampler
+    """
+
+    def __init__(self,
+                 dataset,
+                 num_replicas=None,
+                 rank=None,
+                 shuffle=True,
+                 num_repeats: int = 3,
+                 **kwargs):
+        if num_replicas is None:
+            if not dist.is_available():
+                raise RuntimeError(
+                    'Requires distributed package to be available')
+            num_replicas = dist.get_world_size()
+        if rank is None:
+            if not dist.is_available():
+                raise RuntimeError(
+                    'Requires distributed package to be available')
+            rank = dist.get_rank()
+        if num_repeats < 1:
+            raise ValueError('num_repeats should be greater than 0')
+        self.dataset = dataset
+        self.num_replicas = num_replicas
+        self.rank = rank
+        self.num_repeats = num_repeats
+        self.epoch = 0
+        self.num_samples = int(
+            math.ceil(
+                len(self.dataset) * self.num_repeats / self.num_replicas))
+        self.total_size = self.num_samples * self.num_replicas
+        # self.num_selected_samples = int(math.ceil(len(self.dataset) / self.num_replicas))
+        self.num_selected_samples = int(
+            math.floor(len(self.dataset) // 256 * 256 / self.num_replicas))
+        self.shuffle = shuffle
+
+    def __iter__(self):
+        if self.shuffle:
+            # deterministically shuffle based on epoch
+            g = torch.Generator()
+            g.manual_seed(self.epoch)
+            indices = torch.randperm(len(self.dataset), generator=g)
+        else:
+            indices = torch.arange(start=0, end=len(self.dataset))
+
+        # add extra samples to make it evenly divisible
+        indices = torch.repeat_interleave(
+            indices, repeats=self.num_repeats, dim=0).tolist()
+        padding_size: int = self.total_size - len(indices)
+        if padding_size > 0:
+            indices += indices[:padding_size]
+        assert len(indices) == self.total_size
+
+        # subsample
+        indices = indices[self.rank:self.total_size:self.num_replicas]
+        assert len(indices) == self.num_samples
+
+        return iter(indices[:self.num_selected_samples])
+
+    def __len__(self):
+        return self.num_selected_samples
+
+    def set_epoch(self, epoch):
+        self.epoch = epoch
