@@ -36,6 +36,8 @@ class BEVFormerHead(AnchorFreeHead):
                  num_classes,
                  in_channels,
                  num_query=100,
+                 num_query_one2many=0,
+                 one2many_gt_mul=None,
                  num_reg_fcs=2,
                  with_box_refine=False,
                  as_two_stage=False,
@@ -71,7 +73,6 @@ class BEVFormerHead(AnchorFreeHead):
 
         self.bev_h = bev_h
         self.bev_w = bev_w
-        self.fp16_enabled = False
         self.with_box_refine = with_box_refine
         self.as_two_stage = as_two_stage
         if self.as_two_stage:
@@ -133,13 +134,17 @@ class BEVFormerHead(AnchorFreeHead):
             sampler_cfg = dict(type='PseudoBBoxSampler')
             self.sampler = build_bbox_sampler(sampler_cfg, context=self)
 
-        self.num_query = num_query
+        # for one2many task
+        self.num_query_one2many = num_query_one2many
+        self.num_query_one2one = num_query
+        self.one2many_gt_mul = one2many_gt_mul
+
+        self.num_query = num_query + num_query_one2many if num_query_one2many > 0 else num_query
         self.num_classes = num_classes
         self.in_channels = in_channels
         self.num_reg_fcs = num_reg_fcs
         self.train_cfg = train_cfg
         self.test_cfg = test_cfg
-        self.fp16_enabled = False
         self.loss_cls = build_loss(loss_cls)
         self.loss_bbox = build_loss(loss_bbox)
         self.loss_iou = build_loss(loss_iou)
@@ -279,6 +284,16 @@ class BEVFormerHead(AnchorFreeHead):
                 prev_bev=prev_bev,
             )
         else:
+            # make attn mask for one2many task
+            self_attn_mask = torch.zeros([
+                self.num_query,
+                self.num_query,
+            ]).bool().to(bev_queries.device)
+            self_attn_mask[self.num_query_one2one:,
+                           0:self.num_query_one2one, ] = True
+            self_attn_mask[0:self.num_query_one2one,
+                           self.num_query_one2one:, ] = True
+
             outputs = self.transformer(
                 mlvl_feats,
                 bev_queries,
@@ -292,7 +307,8 @@ class BEVFormerHead(AnchorFreeHead):
                 if self.with_box_refine else None,  # noqa:E501
                 cls_branches=self.cls_branches if self.as_two_stage else None,
                 img_metas=img_metas,
-                prev_bev=prev_bev)
+                prev_bev=prev_bev,
+                attn_mask=self_attn_mask)
 
         bev_embed, hs, init_reference, inter_references = outputs
         hs = hs.permute(0, 2, 1, 3)
@@ -309,20 +325,47 @@ class BEVFormerHead(AnchorFreeHead):
 
             # TODO: check the shape of reference
             assert reference.shape[-1] == 3
-            tmp[..., 0:2] += reference[..., 0:2]
-            tmp[..., 0:2] = tmp[..., 0:2].sigmoid()
-            tmp[..., 4:5] += reference[..., 2:3]
-            tmp[..., 4:5] = tmp[..., 4:5].sigmoid()
-            tmp[..., 0:1] = (
-                tmp[..., 0:1] * (self.pc_range[3] - self.pc_range[0]) +
+            # tmp: torch.Size([1, 900, 10])
+            # tmp[..., 0:2] += reference[..., 0:2]
+            # tmp[..., 0:2] = tmp[..., 0:2].sigmoid()
+            # tmp[..., 4:5] += reference[..., 2:3]
+            # tmp[..., 4:5] = tmp[..., 4:5].sigmoid()
+
+            # tmp[..., 0:1] = (
+            #     tmp[..., 0:1] * (self.pc_range[3] - self.pc_range[0]) +
+            #     self.pc_range[0])
+            # tmp[..., 1:2] = (
+            #     tmp[..., 1:2] * (self.pc_range[4] - self.pc_range[1]) +
+            #     self.pc_range[1])
+            # tmp[..., 4:5] = (
+            #     tmp[..., 4:5] * (self.pc_range[5] - self.pc_range[2]) +
+            #     self.pc_range[2])
+
+            # remove inplace operation, metric may incorrect when using blade
+            tmp_0_2 = tmp[..., 0:2]
+            tmp_0_2_add_reference = tmp_0_2 + reference[..., 0:2]
+            tmp_0_2_add_reference = tmp_0_2_add_reference.sigmoid()
+            tmp_4_5 = tmp[..., 4:5]
+            tmp_4_5_add_reference = tmp_4_5 + reference[..., 2:3]
+            tmp_4_5_add_reference = tmp_4_5_add_reference.sigmoid()
+            tmp_0_1 = tmp_0_2_add_reference[..., 0:1]
+            tmp_0_1_new = (
+                tmp_0_1 * (self.pc_range[3] - self.pc_range[0]) +
                 self.pc_range[0])
-            tmp[..., 1:2] = (
-                tmp[..., 1:2] * (self.pc_range[4] - self.pc_range[1]) +
+            tmp_1_2 = tmp_0_2_add_reference[..., 1:2]
+            tmp_1_2_new = (
+                tmp_1_2 * (self.pc_range[4] - self.pc_range[1]) +
                 self.pc_range[1])
-            tmp[..., 4:5] = (
-                tmp[..., 4:5] * (self.pc_range[5] - self.pc_range[2]) +
+            tmp_4_5_new = (
+                tmp_4_5_add_reference * (self.pc_range[5] - self.pc_range[2]) +
                 self.pc_range[2])
 
+            tmp_2_4 = tmp[..., 2:4]
+            tmp_5_10 = tmp[..., 5:10]
+            tmp = torch.cat(
+                [tmp_0_1_new, tmp_1_2_new, tmp_2_4, tmp_4_5_new, tmp_5_10],
+                dim=-1)
+
             # TODO: check if using sigmoid
             outputs_coord = tmp
             outputs_classes.append(outputs_class)
@@ -333,12 +376,19 @@ class BEVFormerHead(AnchorFreeHead):
 
         outs = {
             'bev_embed': bev_embed,
-            'all_cls_scores': outputs_classes,
-            'all_bbox_preds': outputs_coords,
+            'all_cls_scores':
+            outputs_classes[:, :, :self.num_query_one2one, :],
+            'all_bbox_preds': outputs_coords[:, :, :self.num_query_one2one, :],
             'enc_cls_scores': None,
             'enc_bbox_preds': None,
         }
 
+        if self.num_query_one2many > 0:
+            outs['all_cls_scores_aux'] = outputs_classes[:, :, self.
+                                                         num_query_one2one:, :]
+            outs['all_bbox_preds_aux'] = outputs_coords[:, :, self.
+                                                        num_query_one2one:, :]
+
         return outs
 
     def _get_target_single(self,
@@ -396,6 +446,8 @@ class BEVFormerHead(AnchorFreeHead):
         bbox_weights[pos_inds] = 1.0
 
         # DETR
+        sampling_result.pos_gt_bboxes = sampling_result.pos_gt_bboxes.type_as(
+            bbox_targets)
         bbox_targets[pos_inds] = sampling_result.pos_gt_bboxes
         return (labels, label_weights, bbox_targets, bbox_weights, pos_inds,
                 neg_inds)
@@ -586,6 +638,47 @@ class BEVFormerHead(AnchorFreeHead):
                                               all_gt_bboxes_ignore_list)
 
         loss_dict = dict()
+
+        # for one2many task
+        if 'all_cls_scores_aux' in preds_dicts and self.one2many_gt_mul:
+            all_cls_scores_aux = preds_dicts['all_cls_scores_aux']
+            all_bbox_preds_aux = preds_dicts['all_bbox_preds_aux']
+
+            gt_bboxes_list_aux = []
+            gt_labels_list_aux = []
+            for gt_bboxes, gt_labels in zip(gt_bboxes_list, gt_labels_list):
+                gt_bboxes_list_aux.append(
+                    gt_bboxes.repeat(self.one2many_gt_mul, 1))
+                gt_labels_list_aux.append(
+                    gt_labels.repeat(self.one2many_gt_mul))
+            # for classwise multiply
+            # for gt_bboxes, gt_labels in zip(gt_bboxes_list,gt_labels_list):
+            #     gt_bboxes_aux = []
+            #     gt_labels_aux = []
+            #     for gt_bbox, gt_label in zip(gt_bboxes, gt_labels):
+            #         gt_bboxes_aux += [gt_bbox]*self.one2many_gt_mul[gt_label]
+            #         gt_labels_aux += [gt_label]*self.one2many_gt_mul[gt_label]
+            #     gt_bboxes_list_aux.append(torch.stack(gt_bboxes_aux))
+            #     gt_labels_list_aux.append(torch.stack(gt_labels_aux))
+            all_gt_bboxes_list_aux = [
+                gt_bboxes_list_aux for _ in range(num_dec_layers)
+            ]
+            all_gt_labels_list_aux = [
+                gt_labels_list_aux for _ in range(num_dec_layers)
+            ]
+            losses_cls_aux, losses_bbox_aux = multi_apply(
+                self.loss_single, all_cls_scores_aux, all_bbox_preds_aux,
+                all_gt_bboxes_list_aux, all_gt_labels_list_aux,
+                all_gt_bboxes_ignore_list)
+            loss_dict['loss_cls_aux'] = losses_cls_aux[-1]
+            loss_dict['loss_bbox_aux'] = losses_bbox_aux[-1]
+            num_dec_layer = 0
+            for loss_cls_i, loss_bbox_i in zip(losses_cls_aux[:-1],
+                                               losses_bbox_aux[:-1]):
+                loss_dict[f'd{num_dec_layer}.loss_cls_aux'] = loss_cls_i
+                loss_dict[f'd{num_dec_layer}.loss_bbox_aux'] = loss_bbox_i
+                num_dec_layer += 1
+
         # loss of proposal generated from encode feature map.
         if enc_cls_scores is not None:
             binary_labels_list = [
