@@ -7,6 +7,7 @@ import torch
 import torch.nn as nn
 import torch.nn.functional as F
 
+from easycv.framework.errors import ValueError
 from easycv.models.builder import LOSSES
 from easycv.models.loss.utils import weight_reduce_loss
 
@@ -115,6 +116,7 @@ def binary_cross_entropy(pred,
                          class_weight=None,
                          ignore_index=-100,
                          avg_non_ignore=False,
+                         label_ceil=False,
                          **kwargs):
     """Calculate the binary CrossEntropy loss.
 
@@ -132,11 +134,14 @@ def binary_cross_entropy(pred,
         avg_non_ignore (bool): The flag decides to whether the loss is
             only averaged over non-ignored targets. Default: False.
             `New in version 0.23.0.`
+        label_ceil (bool): When use bce and set label_ceil=True,
+            it will make elements belong to (0, 1] in label change to 1.
+            Default: False.
 
     Returns:
         torch.Tensor: The calculated loss
     """
-    if len(pred.shape) > 1 and pred.shape(1) == 1:
+    if len(pred.shape) > 1 and pred.shape[1] == 1:
         # For binary class segmentation, the shape of pred is
         # [N, 1, H, W] and that of label is [N, H, W].
         # As the ignore_index often set as 255, so the
@@ -162,6 +167,8 @@ def binary_cross_entropy(pred,
             weight = weight * valid_mask
         else:
             weight = valid_mask
+    if label_ceil:
+        label = label.gt(0.0).type(label.dtype)
     # average loss over non-ignored and valid elements
     if reduction == 'mean' and avg_factor is None and avg_non_ignore:
         avg_factor = valid_mask.sum().item()
@@ -234,6 +241,9 @@ class CrossEntropyLoss(nn.Module):
         avg_non_ignore (bool): The flag decides to whether the loss is
             only averaged over non-ignored targets. Default: False.
             `New in version 0.23.0.`
+        label_ceil (bool): When use bce and set label_ceil=True,
+            it will make elements belong to (0, 1] in label change to 1.
+            Default: False.
     """
 
     def __init__(self,
@@ -243,10 +253,16 @@ class CrossEntropyLoss(nn.Module):
                  class_weight=None,
                  loss_weight=1.0,
                  loss_name='loss_ce',
-                 avg_non_ignore=False):
+                 avg_non_ignore=False,
+                 label_ceil=False):
         super(CrossEntropyLoss, self).__init__()
         assert (use_sigmoid is False) or (use_mask is False)
         self.use_sigmoid = use_sigmoid
+        if label_ceil:
+            if not use_sigmoid:
+                raise ValueError(
+                    '‘label_ceil’ is supported only when ‘use_sigmoid’ is true. If not use bce, please set ‘label_ceil’=False'
+                )
         self.use_mask = use_mask
         self.reduction = reduction
         self.loss_weight = loss_weight
@@ -266,6 +282,7 @@ class CrossEntropyLoss(nn.Module):
         else:
             self.cls_criterion = cross_entropy
         self._loss_name = loss_name
+        self.label_ceil = label_ceil
 
     def extra_repr(self):
         """Extra repr."""
@@ -289,16 +306,29 @@ class CrossEntropyLoss(nn.Module):
         else:
             class_weight = None
         # Note: for BCE loss, label < 0 is invalid.
-        loss_cls = self.loss_weight * self.cls_criterion(
-            cls_score,
-            label,
-            weight,
-            class_weight=class_weight,
-            reduction=reduction,
-            avg_factor=avg_factor,
-            avg_non_ignore=self.avg_non_ignore,
-            ignore_index=ignore_index,
-            **kwargs)
+        if self.use_sigmoid:
+            loss_cls = self.loss_weight * self.cls_criterion(
+                cls_score,
+                label,
+                weight,
+                class_weight=class_weight,
+                reduction=reduction,
+                avg_factor=avg_factor,
+                avg_non_ignore=self.avg_non_ignore,
+                ignore_index=ignore_index,
+                label_ceil=self.label_ceil,
+                **kwargs)
+        else:
+            loss_cls = self.loss_weight * self.cls_criterion(
+                cls_score,
+                label,
+                weight,
+                class_weight=class_weight,
+                reduction=reduction,
+                avg_factor=avg_factor,
+                avg_non_ignore=self.avg_non_ignore,
+                ignore_index=ignore_index,
+                **kwargs)
         return loss_cls
 
     @property
