@@ -9,6 +9,8 @@ from .feature_extractor import (TorchFaceAttrExtractor,
                                 TorchFaceFeatureExtractor,
                                 TorchFeatureExtractor)
 from .hand_keypoints_predictor import HandKeypointsPredictor
+from .ocr import (OCRClsPredictor, OCRDetPredictor, OCRPredictor,
+                  OCRRecPredictor)
 from .pose_predictor import (TorchPoseTopDownPredictor,
                              TorchPoseTopDownPredictorWithDetector)
 from .segmentation import Mask2formerPredictor, SegmentationPredictor
