@@ -6,6 +6,9 @@ import numpy as np
 import torch
 from PIL import Image
 
+from easycv.framework.errors import ValueError
+from easycv.thirdparty.face_align import glint360k_align
+from easycv.thirdparty.mtcnn import FaceDetector
 from .base import Predictor
 from .builder import PREDICTORS
 
@@ -14,13 +17,6 @@ try:
 except:
     from .interface import PredictorInterface
 
-try:
-    from thirdparty.mtcnn import FaceDetector
-    from thirdparty.face_align import glint360k_align
-except:
-    from easycv.thirdparty.mtcnn import FaceDetector
-    from easycv.thirdparty.face_align import glint360k_align
-
 
 @PREDICTORS.register_module()
 class TorchFeatureExtractor(PredictorInterface):
@@ -522,7 +518,7 @@ class TorchFaceAttrExtractor(PredictorInterface):
                             neck_output.device)
                         neck_output = (distribute * neck_output).sum(dim=1)
                     else:
-                        raise Exception(
+                        raise ValueError(
                             'TorchFaceAttrExtractor for neck %d only support attr_method softmax/distributed sum'
                             % (neck_idx))
                         neck_output = torch.argmax(neck_output, dim=1)
